SystemOrganization addCategory: #'GT-Debugger'!

!SaveDebugAction methodsFor: '*GT-Debugger' stamp: 'AndreiChis 6/18/2013 17:23'!
asGlamourAction

	^ super asGlamourAction
			action: [ :textPresentation | 
				self sourceCode: textPresentation text.
				self execute ]! !

Debugger subclass: #GTProxyDebugger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger'!

!GTProxyDebugger commentStamp: 'TudorGirba 11/2/2012 22:04' prior: 0!
GTProxyDebugger is a simple extension of the existing debugger that adds a button for "debug glamorously" using the GTDebugger in the regular debugger.

The goal is to provide the possibility of having the GTDebugger next to the classic debugger until the inevitable errors are ironed out.!

!GTProxyDebugger class methodsFor: 'class initialization' stamp: 'AndreiChis 11/1/2012 15:13'!
initialize
	super initialize.

	"Add ourselves to registry. See [Smalltalk tools]" 
	Smalltalk tools register: self as: #debugger! !

!GTProxyDebugger methodsFor: 'accessing' stamp: 'AndreiChis 11/25/2012 12:52'!
debuggerClass
	^ GTDebugger ! !

!GTProxyDebugger methodsFor: 'initialize' stamp: 'TudorGirba 11/1/2012 21:23'!
preDebugButtonQuads
	| buttons |
	self isPostMortem 
		ifTrue: [ ^ self postMortemPreDebugButtonQuads ].
	^ {
		{'Proceed' translated.	#proceed. 	#blue. 	'Continue execution' translated}.
		{'Abandon' translated.	#abandon. 	#black.	'Abandon this execution by closing this window' translated}.
		{'Debug'    		translated.		#debug.		#red. 	'Bring up a debugger' translated}.
		{'Debug glamorously' translated. #spawnGTDebugger. #red. 'Bring up the Glamorous Debugger' translated}
	}! !

!GTProxyDebugger methodsFor: 'notifier menu' stamp: 'AndreiChis 1/15/2013 12:14'!
spawnGTDebugger
	| topView |
	topView := self topView.
	topView model: nil.
	"so close won't release me."
	self breakDependents.
	topView delete.
	
	self debuggerClass debugSession: ((self debuggerClass 
		spanNewSessionForContext: self interruptedContext fromProcess: self interruptedProcess)
			errorWasInUIProcess: errorWasInUIProcess) 
! !

!DebugAction methodsFor: '*GT-Debugger' stamp: 'AndreiChis 6/17/2013 16:43'!
asGlamourAction

	^ ((GLMGenericAction new) 
			action: [ self execute ]; 
			icon: self icon;
			title: self label;
			yourself)! !

!DebugContext methodsFor: '*gt-debugger' stamp: 'AndreiChis 6/5/2013 14:53'!
gtDebuggerRetrieveVariableValuePairs
	|fieldList|

	fieldList := OrderedCollection new.
	fieldList 
		add: ('_self' ->  context receiver);
	      addAll: (context receiver class allInstVarNames collect: [:each | each -> (context receiver instVarNamed: each) ]).
	
	fieldList add: ('_thisContext' ->  context).
	 context == nil ifTrue: [^fieldList].
	fieldList 
		add: ('_stack top' -> ( context stackPtr > 0 ifTrue: [ context top])).
		
	(context tempNames size > 0) ifTrue: [
		fieldList addAll: ((1 to: context tempNames size) collect: [ :index || key value |
			key := context tempNames at: index.
			value := [context debuggerMap namedTempAt: index in: context] 
				on: Error 
				do: [value := (Text string: 'error obtaining field value' attribute: TextColor red) asString ].
			key -> value ] ) ].
	
	^ fieldList! !

!DebugSession class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 6/18/2013 17:50'!
gtChangeDebuggerActionsFor: aDebugger
	<gtContextDebuggingAction>
	
	^ (aDebugger debuggersForContext: aDebugger selectedContext) collect: [ :aDebuggerClass | 
		ChangeDebuggerAction new
			debuggerClass: aDebuggerClass
		]
		! !

!DebugSession class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 6/18/2013 17:39'!
gtCodeDebugActionFor: aDebugger
	<gtCodeDebuggingActions>
	
	^ {  
		WhereIsDebugAction new order: 5.
		RunToHereDebugAction new order: 10.
		SaveDebugAction new order: 15.
	}! !

!DebugSession class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 6/18/2013 17:40'!
gtDebuggerActionsForPragma: aSymbol for: aDebugger
	| pragmas actions |
	pragmas := Pragma 
				allNamed: aSymbol
				from: self  class
				to: DebugSession class.
	
	actions := OrderedCollection new.
	pragmas do: [ :eachPragma |
		actions addAll: 
			(self perform: eachPragma selector with: aDebugger) ].
		
	^ actions sort: [ :action1 :action2 | action1 order < action2 order ]! !

!DebugSession class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 6/18/2013 17:39'!
gtDynamicStackDebugActionFor: aDebugger
	<gtContextDebuggingAction>
	
	^ { 
		CreateMessageDebugAction new
			order: 50.
	 }! !

!DebugSession class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 6/18/2013 17:39'!
gtProcessDebugActionFor: aDebugger
	<gtDebuggingActions>

	^ { 
		ResumeDebugAction new order: 5.
		RestartDebugAction new order: 10.
	 }! !

!DebugSession class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 6/18/2013 17:38'!
gtStackDebugActionFor: aDebugger
	<gtDebuggingActions>
	
	^ { 
		StackDebugAction new
			label: 'Step Into';
			action: #stepInto:;
			icon: GLMUIThemeExtraIcons glamorousInto;
			order: 20.
			
		StackDebugAction new
			label: 'Step Over';
			action: #stepOver:;
			icon: GLMUIThemeExtraIcons glamorousOver;
			order: 25.
			
		StackDebugAction new
			label: 'Step Through';
			action: #stepThrough:;
			icon: GLMUIThemeExtraIcons glamorousThrough;
			order: 30.
	 }! !

GLMCompositePresentation subclass: #GTDebugger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger'!

!GTDebugger commentStamp: '<historical>' prior: 0!
GTDebugger is a browser that provides Smalltalk debugging functionality.

An example: 
[ GTDebugger debugProcess: Processor activeProcess.
	Processor activeProcess suspend.
	#(1 2 3 4 5) do: [:each | each asString]
] fork.!

!GTDebugger class methodsFor: 'instance creation' stamp: 'AndreiChis 4/2/2013 14:49'!
debugBasedOnSession: aSession

	^ self debugSession: ((self spanNewSessionForContext: aSession context fromProcess: aSession process)
		errorWasInUIProcess: aSession errorWasInUIProcess)! !

!GTDebugger class methodsFor: 'instance creation' stamp: 'AndreiChis 1/28/2013 16:39'!
debugContext: aContext fromProcess: aProcess

	^ self debugSession: (self spanNewSessionForContext: aContext fromProcess: aProcess)! !

!GTDebugger class methodsFor: 'instance creation' stamp: 'AndreiChis 1/28/2013 16:39'!
debugProcess: aProcess

	^ self debugContext: aProcess suspendedContext fromProcess: aProcess! !

!GTDebugger class methodsFor: 'instance creation' stamp: 'AndreiChis 1/28/2013 16:39'!
debugSession: aSession

	<primitive: 19> "simulation guard"

	"WorldState addDeferredUIMessage: ["
		^ self openOn: aSession 
	"]"! !

!GTDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 3/26/2013 15:33'!
defaultTitle
	^ 'Glamorous Debugger'! !

!GTDebugger class methodsFor: 'testing' stamp: 'AndreiChis 3/25/2013 14:42'!
handlesContext: aContext

	^ true! !

!GTDebugger class methodsFor: 'class initialization' stamp: 'AndreiChis 6/19/2013 15:40'!
initialize 

	"Add ourselves to registry. See [Smalltalk tools]" 
	Smalltalk tools registerDebugger: self withRank: 3.! !

!GTDebugger class methodsFor: 'instance creation' stamp: 'TudorGirba 11/25/2012 23:00'!
openOn: anObject 
	^ self new openOn: anObject! !

!GTDebugger class methodsFor: 'private' stamp: 'AndreiChis 1/13/2013 20:19'!
sessionClass

	^ DebugSession ! !

!GTDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 1/15/2013 12:07'!
spanNewSessionForContext: aContext fromProcess: aProcess

	^ self sessionClass process: aProcess context: aContext ! !

!GTDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 1/28/2013 14:33'!
spanNewSessionForProcess: aProcess

	^ self spanNewSessionForContext: aProcess suspendedContext fromProcess: aProcess! !

!GTDebugger methodsFor: 'actions' stamp: 'AndreiChis 3/27/2013 11:56'!
actOnBrowserClosing: ann
	self session ifNotNil: [ :aSession | aSession terminate ]! !

!GTDebugger methodsFor: 'building actions' stamp: 'AndreiChis 6/19/2013 12:18'!
actionsForPragmas: aSymbolsCollection
	| allActions |

	allActions := self allActionsForPragmas: aSymbolsCollection.
	
	allActions do: [ :aDebugAction |
		aDebugAction debugger: self ]. "should be removed"

	^ (allActions select: [ :aDebugAction | 
			aDebugAction appliesToContextCurrentContext ]) 
				collect: [ :aDebugAction | aDebugAction asGlamourAction  ]



	
! !

!GTDebugger methodsFor: 'building actions' stamp: 'AndreiChis 6/18/2013 17:41'!
allActionsForPragmas: aSymbolsCollection
	
	^ aSymbolsCollection 
		inject: OrderedCollection new
		into: [ :currentActions :aSymbol | 
			currentActions 
				addAll: (self session class gtDebuggerActionsForPragma: aSymbol for: self);
				yourself ].


	
! !

!GTDebugger methodsFor: 'accessing' stamp: 'AndreiChis 11/28/2012 09:52'!
browser 
	^ self presentations first! !

!GTDebugger methodsFor: 'actions' stamp: 'AndreiChis 6/17/2013 16:31'!
close

	self browser close! !

!GTDebugger methodsFor: 'building' stamp: 'AndreiChis 6/20/2013 12:03'!
compose
	self title: self debuggerTitle.
	self tabulator with: [:browser | 
		self debuggerStructureIn: browser.
		self debuggerTransmissionsIn: browser ].
	self registerActions.
! !

!GTDebugger methodsFor: 'actions' stamp: 'AndreiChis 4/1/2013 16:19'!
debug: aContext using: aDebuggerClass
	| currentSession  |
	
	currentSession := self session.
	self detachSession.
	self browser close.
	currentSession updateContextTo: aContext.
	aDebuggerClass debugBasedOnSession: currentSession! !

!GTDebugger methodsFor: 'building' stamp: 'AndreiChis 6/20/2013 12:01'!
debuggerStructureIn: browser
	
	browser
		row: [ :row | 
					row
						column: #stack;
						column: #code ];
		row: #inspector.
		! !

!GTDebugger methodsFor: 'accessing' stamp: 'AndreiChis 3/26/2013 15:35'!
debuggerTitle
	
	^ self class defaultTitle ! !

!GTDebugger methodsFor: 'building' stamp: 'AndreiChis 6/20/2013 12:03'!
debuggerTransmissionsIn: browser

	browser transmit
		to: #stack;
		andShow: [ :composite | self stackIn: composite ].
	browser transmit
		fromOutsidePort: #entity;
		to: #stack port: #selection;
		transformed: [:session | session context].
	browser transmit
		from: #stack port: #selection;
		fromOutsidePort: #entity;
		to: #stack port: #context;
		transformed: [ :context :session | session createModelForContext: context ].
	browser transmit
		from: #stack port: #selection;
		to: #stack port: #pc;
		transformed: #pc.
	browser transmit
		from: #stack port: #selection;
		to: #stack port: #fakePort;
		transformed: [ :value | "self updateStackToolbar"].
	browser transmit
		from: #stack port: #context;
		fromOutsidePort: #entity;
		to: #code;
		andShow: [ :composite | self methodCodeIn: composite ].
	browser transmit
		from: #stack;
		fromOutsidePort: #entity;
		from: #stack port: #pc;
		to: #code port: #selectionInterval;
		transformed: [ :context :session :pc | session selectedCodeRangeForContext: context ].
	browser transmit
		from: #stack port: #context;
		to: #inspector;
		andShow: [ :composite | self inspectorForContextAndSelfIn: composite ] 
! !

!GTDebugger methodsFor: 'accessing' stamp: 'AndreiChis 4/2/2013 15:44'!
debuggersForContext: aContext
	"Ungly check to make sure the debugger will work until everything is in the image"
	(Smalltalk tools respondsTo: #debuggersForContext:excluding:) 
		ifTrue: [ ^ (Smalltalk tools debuggersForContext: aContext excluding: self class)]
		ifFalse: [ ^ OrderedCollection new ]

	! !

!GTDebugger methodsFor: 'actions' stamp: 'AndreiChis 3/27/2013 10:35'!
detachSession

	(self browser pane port: #entity) silentValue: nil! !

!GTDebugger methodsFor: 'intialization' stamp: 'AndreiChis 3/27/2013 11:59'!
errorWasInUIProcess: aBoolean

	self session errorWasInUIProcess: aBoolean! !

!GTDebugger methodsFor: 'building' stamp: 'AndreiChis 6/17/2013 17:58'!
inspectorForContextAndSelfIn: composite
	composite dynamic
		display: [ :each | 
			"We use the first actual finder because need to send showFirst: to it"
			GTInspector new first
				noTitle;
				noActions;
				showFirst: [ :a | a custom: (GTDebuggerVariablesBrowser new startOn: each) ];
				startOn: each ]! !

!GTDebugger methodsFor: 'building actions' stamp: 'AndreiChis 6/18/2013 17:25'!
installCodeActionsFor: aPresentation

	aPresentation dynamicActions: [ :presentation |
		self actionsForPragmas: {#gtCodeDebuggingActions}  ]! !

!GTDebugger methodsFor: 'building actions' stamp: 'AndreiChis 6/19/2013 15:03'!
installDebuggingActionsFor: aPresentation

	aPresentation dynamicActions: [ :presentation |
		self actionsForPragmas: {#gtDebuggingActions}  ].
	
	aPresentation dynamicActionsOnSelection: [ :presentation |
		self actionsForPragmas: {#gtContextDebuggingAction} ]! !

!GTDebugger methodsFor: 'accessing' stamp: 'AndreiChis 6/19/2013 19:07'!
interruptedContext

	^ self session context! !

!GTDebugger methodsFor: 'building' stamp: 'AndreiChis 6/19/2013 18:57'!
methodCodeIn: composite
	composite smalltalkCode
		doItReceiver: [:contextModel | contextModel receiver ];
		title: 'Source';
		format: #source;
		smalltalkClass: [ :contextModel | contextModel selectedClass ];
		with: [ :code | self installCodeActionsFor: code ]! !

!GTDebugger methodsFor: 'opening' stamp: 'AndreiChis 3/26/2013 14:43'!
openFullNoSuspendLabel: aString
	"TODO: do not ignore the title"
	
	self open.! !

!GTDebugger methodsFor: 'opening' stamp: 'AndreiChis 4/3/2013 00:34'!
openWith: aRenderer
	|window|
	
	window := super openWith: aRenderer.
	window 
		width: 1000; 
		height: 600.
	^ window! !

!GTDebugger methodsFor: 'intialization' stamp: 'AndreiChis 3/27/2013 12:00'!
process: aProcess controller: aController context: aContext

	self session: (self class spanNewSessionForContext: aContext fromProcess: aProcess)! !

!GTDebugger methodsFor: 'building' stamp: 'AndreiChis 6/20/2013 11:57'!
registerActions

	self on: GLMBrowserClosing send: #actOnBrowserClosing: to: self.
	self on: DebugActionExecuted send: #updateStackPane to: self.
! !

!GTDebugger methodsFor: 'private' stamp: 'AndreiChis 6/19/2013 19:09'!
selectTopOfStack: aContext
"	| stackPane |
	stackPane := self browser paneNamed: #stack.
	(stackPane port: #selection) value: aContext.
	(stackPane port: #pc) value: aContext pc.
	stackPane update."! !

!GTDebugger methodsFor: 'accessing' stamp: 'AndreiChis 11/28/2012 09:54'!
selectedContext
	^ ((self browser paneNamed: #stack) port: #selection) value! !

!GTDebugger methodsFor: 'private' stamp: 'AndreiChis 1/13/2013 19:27'!
selectionInterval
	^ ((self browser paneNamed: #code) port: #selectionInterval) value! !

!GTDebugger methodsFor: 'private' stamp: 'AndreiChis 1/13/2013 20:07'!
selectionInterval: anInterval
	^ ((self browser paneNamed: #code) port: #selectionInterval) value: anInterval! !

!GTDebugger methodsFor: 'accessing' stamp: 'AndreiChis 3/27/2013 11:58'!
session
	^ (self browser pane port: #entity) value! !

!GTDebugger methodsFor: 'accessing' stamp: 'AndreiChis 3/27/2013 11:58'!
session: anObject

	^ (self browser pane port: #entity) value: anObject! !

!GTDebugger methodsFor: 'building' stamp: 'AndreiChis 6/19/2013 19:03'!
stackIn: composite
	composite list	
		"updateOn: DebugActionExecuted from: [ self announcer ];"
		title: 'Stack';
		"showOnly: 10;"
		display: #stack;
		format: [ :each | String streamContents: [:stream | each printDebugOn: stream ] ];
		with: [ :list | self installDebuggingActionsFor: list ]! !

!GTDebugger methodsFor: 'private' stamp: 'AndreiChis 2/7/2013 17:03'!
updateCodePane 
	| codePane |
	
	codePane := self browser paneNamed: #code.
	codePane update.
	self updateSelectionInterval.! !

!GTDebugger methodsFor: 'private' stamp: 'AndreiChis 6/19/2013 19:11'!
updateInspectorPane
! !

!GTDebugger methodsFor: 'private' stamp: 'AndreiChis 3/27/2013 12:00'!
updateSelectionInterval

	self selectionInterval: (self session pcRangeForContext:  self selectedContext )! !

!GTDebugger methodsFor: 'private' stamp: 'AndreiChis 6/19/2013 20:16'!
updateStackPane
	| stackPane |
	
	stackPane := self browser paneNamed: #stack.
	(stackPane port: #selection) silentValue: nil.
	(stackPane port: #selection) value: self interruptedContext.
	stackPane update.! !

!GTDebugger methodsFor: 'events' stamp: 'AndreiChis 1/15/2013 15:08'!
windowIsClosing 
	self announce: GLMBrowserClosing new! !

GLMCompositePresentation subclass: #GTDebuggerVariablesBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger'!

!GTDebuggerVariablesBrowser commentStamp: 'TudorGirba 11/2/2012 22:06' prior: 0!
GTDebuggerVariablesBrowser is a special browser that is used in the first pane of the inspector that is embedded in the debugger. Its main intention is to provide a unified list of variables that come from:
- the instance variables of the receiver
- the local variables from the context!

!GTDebuggerVariablesBrowser methodsFor: 'building' stamp: 'TudorGirba 11/25/2012 22:57'!
compose  
	self title: [:each | GTInspector new printObjectAsAnItem: each receiver ].
	self tabulator with: [:browser | 
		browser 
			morphicAct: [:anotherComposite | anotherComposite entity receiver browse]
				icon: GLMUIThemeExtraIcons glamorousBrowse 
				entitled: 'Browse'.
		browser row: #variables; row: #evaluator.
		browser transmit to: #variables; andShow: [:a | 
			self variablesIn: a ].
		browser transmit to: #evaluator; andShow: [:a | 
			self evaluatorIn: a ].
		browser transmit from: #variables; toOutsidePort: #selection.
		browser transmit from: #evaluator; toOutsidePort: #selection ]! !

!GTDebuggerVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 10/30/2012 21:03'!
evaluatorIn: composite
	(composite smalltalkCode)
		doItReceiver: #yourself;
		display: [ :model | [model receiver printString asSmalltalkComment, String cr, 'self'] on: Error do: ['"Error printing. Try self printString to debug"'] ];
		selectionPopulate: #selection
			on: $o
			entitled: 'Do it and open (o)'
			with: [ :text :model | Compiler evaluate: text selectedText for: model receiver logged: false ]! !

!GTDebuggerVariablesBrowser methodsFor: 'private building' stamp: 'TudorGirba 11/2/2012 22:10'!
variablesIn: composite
	composite table
		showOnly: 50;
		display: [ :contextModel | contextModel gtDebuggerRetrieveVariableValuePairs ];
		sorted: [:variablePair1 :variablePair2 | variablePair1 key < variablePair2 key];
		column: 'Variable' evaluated: [:assoc | 
			[ Text fromString: assoc key asString ]
						on: Error
						do: [ Text string: 'error printing' attribute: TextColor red ] ];
		column: 'Value' evaluated: [:assoc | 
			[ Text fromString: (assoc value printString contractTo: 50) ]
						on: Error
						do: [ Text string: 'error printing' attribute: TextColor red ] ];
"		icon: [ :each |
			each value class theNonMetaClass browserIcon
				ifNotNil: [ OBMorphicIcons iconNamed: each value class theNonMetaClass browserIcon ] ];"
		send: #value;
		morphicSelectionAct: [:each | each value browse ] 
			icon: GLMUIThemeExtraIcons glamorousBrowse 
			on: $b 
			entitled: 'Browse'! !

!PreDebugWindow methodsFor: '*gt-debugger' stamp: 'TudorGirba 11/1/2012 21:23'!
spawnGTDebugger
	model spawnGTDebugger! !
GTDebugger initialize!
GTProxyDebugger initialize!
